module OO
  class Configuration
    attr_reader :license_max_depth, :default_license_keys,
                :license_missing_reaction, :license_checkers_default_user,
                :license_checkers_default_key

    def initialize
      @license_max_depth = 4
      @default_license_keys = []
      @license_missing_reaction =
        -> { :license_missing_reaction_not_configured }

      @license_checkers_default_user =
        -> { :license_checkers_default_user_not_configured }

      @license_checkers_default_key =
        -> { :license_checkers_default_key_not_configured }
    end

    def license_max_depth=(depth)
      raise Errors::LicenseMaxDepthInvalid unless depth.is_a?(Fixnum) &&
                                                  depth > 0
      @license_max_depth = depth
    end

    def default_license_keys=(license_keys)
      raise Errors::DefaultLicenseKeysInvalid unless license_keys.is_a?(Array)
      @default_license_keys = license_keys
    end

    def license_missing_reaction=(action)
      raise Errors::LicenseMissingReactionInvalid unless action.is_a?(Proc)
      @license_missing_reaction = action
    end

    def license_checkers_default_user=(action)
      raise Errors::LicenseCheckersDefaultUserInvalid unless action.is_a?(Proc)
      @license_checkers_default_user = action
    end

    def license_checkers_default_key=(action)
      raise Errors::LicenseCheckersDefaultKeyInvalid unless action.is_a?(Proc)
      @license_checkers_default_key = action
    end
  end
end
