module OO
  class License
    class << self
      def separator
        '.'
      end

      def mask_mark
        '*'
      end

      def max_depth
        OO.configuration.license_max_depth
      end

      def masks
        masks = {}
        max_depth.times do |t|
          masks["#{t + 1}"] = mask_mark + "#{separator}#{mask_mark}" * t
        end
        masks
      end
    end

    attr_reader :key

    alias_method :to_s, :key

    def initialize(attrs = {})
      @key = attrs.fetch(:key)
    end

    def masks
      masks = []
      gen_masks = genre_masks
      current = self
      while current.key != mask_mark
        gen_masks.each { |mask| masks << current.mask_key(mask) }
        current = current.parent
      end
      (masks + gen_masks).uniq
    end

    def parent
      return self.class.new(key: mask_mark) if depth == 1
      self.class.new(key: to_a[0..-2].join(separator))
    end

    def to_a
      key.split(separator)
    end

    def depth
      to_a.size
    end

    def separator
      self.class.separator
    end

    def mask_mark
      self.class.mask_mark
    end

    def genre_masks
      (depth..self.class.masks.size).map { |n| self.class.masks[n.to_s] }
    end

    def mask_key(mask)
      mask = mask.split(separator)
      mask[0..depth - 1] = *to_a
      mask.join(separator)
    end
  end
end
