module OO
  module LicenseCheckers
    def self.included(base)
      base.class_eval do
        helper_method(:licensed?) if defined?(Rails)

        def licensed?(attrs = {})
          user = attrs.fetch(:user, license_checkers_default_user)
          key = attrs.fetch(:to, license_checkers_default_key)
          user.licensed?(to: key)
        end

        def check_license!(attrs = {})
          user = attrs.fetch(:user, license_checkers_default_user)
          key = attrs.fetch(:to, license_checkers_default_key)
          license_missing_reaction.call(key, user) unless licensed?(user: user,
                                                                    to: key)
        end

        private

        def license_checkers_default_user
          OO.configuration.license_checkers_default_user.call(self)
        end

        def license_checkers_default_key
          OO.configuration.license_checkers_default_key.call(self)
        end

        def license_missing_reaction
          OO.configuration.license_missing_reaction
        end
      end
    end
  end
end
